//  Unit of TSP Solver
//  Created by GrMikeD, 2000-2003
//  http://kickme.to/GrMikeD
//  GrMikeD@Freemail.gr
unit Algorithm;

interface

uses
  Graph, Counter, Classes;

type
  TFoundEvent = procedure(Path: string; Cost: Integer) of object;

type
  TEndEvent = procedure(Success: Boolean; Time: string; Flop: Integer) of object;

type
  TAlgorithm = class (TThread)
  private
    fCounter: TCounter;
    function Start: Boolean;
  protected
    pGraph: TGraph;
    pTime: string;
    pFlop: Integer;
    pSize: Integer;
    pPath: String;
    pCost: Integer;
    pP: array of Integer;
    pForceStop: Boolean;
    constructor Create(Graph: TGraph; Suspended: Boolean = False);
    function Run: Boolean; virtual; abstract;
    procedure FireFound;
    procedure SetPath;
    procedure Execute; override;
  public
    OnFound: TFoundEvent;
    OnEnd: TEndEvent;
    procedure ForceStop;
    destructor Destroy; override;
  end;

implementation

{ TAlgorithm }

uses
  SysUtils;

constructor TAlgorithm.Create(Graph: TGraph; Suspended: Boolean = False);
begin
  inherited Create(Suspended);
  FreeOnTerminate:= True;
  pForceStop:= False;
  pGraph:= Graph;
  pTime:= '0';
  pFlop:= 0;
  pSize:= pGraph.Size;
  pCost:= 0;
  pPath:= '';
  SetLength(pP, pSize + 1);
  fCounter:= TCounter.Create;
end;

destructor TAlgorithm.Destroy;
begin
  fCounter.Free;
  pP:= nil;
  inherited Destroy;
end;

function TAlgorithm.Start: Boolean;
begin
  fCounter.StartTimeCount;
  Result:= Run;
  pTime:= fCounter.ElapsedSecs;
end;

procedure TAlgorithm.FireFound;
var
  Elapsed: Int64;
begin
  Elapsed:= fCounter.ElapsedCounts;
  if (Assigned(OnFound)) then
    OnFound(pPath, pCost);
  fCounter.StartTimeCount(Elapsed);
end;

procedure TAlgorithm.SetPath;
var
  j: Integer;
begin
  pPath:= '';
  for j:=0 to pSize do
    pPath:= pPath + 'a' + IntToStr(pP[j] + 1);
end;

procedure TAlgorithm.Execute;
var
  b: Boolean;
begin
  b:= Start;
  if (Assigned(OnEnd)) then
    OnEnd(b, pTime, pFlop);
end;

procedure TAlgorithm.ForceStop;
begin
  pForceStop:= True;
end;

end.
