//  Unit of TSP Solver
//  Created by GrMikeD, 2000-2003
//  http://kickme.to/GrMikeD
//  GrMikeD@Freemail.gr
unit Exhaustive;

interface

uses
  Algorithm, Graph;

type
  TExhaustive = class (TAlgorithm)
  private
    fA: array of integer;
    fError: Boolean;
    fP1, fP2: array of integer;
    procedure InitValues;
    procedure Swap(var s1, s2: Integer);
    function Permute(i: Integer): Boolean;
    function Process: Boolean;
    function Compare: Boolean;
    function Calculate(P: array of Integer; out Cost: Integer): Boolean;
    procedure SetActivePath(P: array of Integer; Cost: Integer);
  public
    function Run: Boolean; override;
    constructor Create(Graph: TGraph; Suspended: Boolean = False);
    destructor Destroy; override;
  end;

implementation

uses
  SysUtils, Dialogs, Error;

{ TExhaustive }

constructor TExhaustive.Create(Graph: TGraph; Suspended: Boolean);
begin
  inherited Create(Graph, Suspended);
  fError:= False;
  SetLength(fA, pSize);
  SetLength(fP1, pSize + 1);
  SetLength(fP2, pSize + 1);
  InitValues;
end;

destructor TExhaustive.Destroy;
begin
  fA:= nil;
  fP1:= nil;
  fP2:= nil;
  inherited Destroy;
end;

procedure TExhaustive.InitValues;
var
  i: Integer;
begin
  for i:= 0 to pSize - 1 do
    fA[i]:= i;
end;

function TExhaustive.Calculate(P: array of Integer; out Cost: Integer): Boolean;
var
  j: Integer;
begin
  Cost:= 0;
  Result:= False;
  for j:= 0 to pSize - 1 do
  begin
    Inc(pFlop);
    if (pGraph.GetValue(P[j], P[j+1]) = -1) then
    begin
      Result:= True;
      Break;
    end
    else
      Cost:= Cost + pGraph.GetValue(P[j], P[j+1]);
  end;
end;

procedure TExhaustive.SetActivePath(P: array of Integer; Cost: Integer);
var
  j: Integer;
begin
  for j:= 0 to pSize do
    pP[j]:= P[j];
  pCost:= Cost;
  SetPath;
end;

function TExhaustive.Compare: Boolean;
var
  Cost: Integer;
  Error: Boolean;
begin
  Result:= True;
  if pForceStop then
    Exit;
  Error:= Calculate(fP2, Cost);
  Inc(pFlop);
  if (not Error) and (fError or ((not fError) and (Cost < pCost))) then
  begin
    SetActivePath(fP2, Cost);
    Synchronize(FireFound);
  end
end;

function TExhaustive.Process: Boolean;
var
  j: Integer;
begin
  Result:= True;
  if (fP1[1] = 0) then
  begin
    for j:= 0 to pSize - 1 do
      fP1[j]:= fA[j];
    fP1[pSize]:= fP1[0];
    fError:= Calculate(fP1, pCost);
    if (fError) then
    begin
      pPath:= E_PATH;
      pCost:= 0;
    end
    else
    begin
      SetActivePath(fP1, pCost);
    end;
    Synchronize(FireFound);
  end
  else
  begin
    for j:=0 to pSize - 1 do
      fP2[j]:= fA[j];
    fP2[pSize]:= fP2[0];
    Result:= Compare;
  end;
end;

function TExhaustive.Permute(i: Integer): Boolean;
var
  j: Integer;
begin
  if (pForceStop) then
  begin
    Result:= False;
    Exit;
  end;
  if (i = pSize - 1) then
    Result:= Process
  else
  begin
    Result:= Permute(i + 1);
    for j:= i + 1 to pSize - 1 do
    begin
      Swap(fA[i], fA[j]);
      Result:= Permute(i + 1);
      Swap(fA[i], fA[j]);
    end;
  end;
end;

function TExhaustive.Run: Boolean;
begin
  Result:= Permute(1);
end;

procedure TExhaustive.Swap(var s1, s2: Integer);
var
  j: Integer;
begin
 j:= s1;
 s1:= s2;
 s2:= j;
end;

end.
