//  Unit of TSP Solver
//  Created by GrMikeD, 2000-2003
//  http://kickme.to/GrMikeD
//  GrMikeD@Freemail.gr
unit Heuristic;

interface

uses
  Algorithm, Graph;

type
  THeuristic = class (TAlgorithm)
  private
    fQ: array of array of Integer;
    procedure InitValues;
    function SmallerDistance(const Index: Integer; out NextIndex: Integer): Boolean;
  public
    function Run: Boolean;  override;
    constructor Create(Graph: TGraph; Suspended: Boolean = False);
    destructor Destroy; override;
  end;

implementation

uses
  SysUtils, Error;

{ THeuristic }

constructor THeuristic.Create(Graph: TGraph; Suspended: Boolean = False);
begin
  inherited Create(Graph, Suspended);
  SetLength(fQ, pSize, pSize);
  InitValues;
end;

procedure THeuristic.InitValues;
var
  i, j: Integer;
begin
  for i:= 0 to pSize - 1 do
    for j:= 0 to pSize - 1 do
      fQ[i, j]:= pGraph.GetValue(i, j);
end;

destructor THeuristic.Destroy;
begin
  fQ:= nil;
  inherited Destroy;
end;

function THeuristic.SmallerDistance(const Index: Integer; out NextIndex: Integer): Boolean;
var
  i: Integer;
begin
  NextIndex:= 0;
  Result:= False;
  for i:= 0 to pSize - 1 do
  begin
    if (i <> Index) then
    begin
      Inc(pFlop);
      if ((fQ[Index, i] <> -1) and ((fQ[Index, NextIndex] = -1) or (fQ[Index, i] <= fQ[Index, NextIndex]))) then
      begin
        NextIndex:= i;
        Result:= True;
      end;
    end;
  end;
  fQ[Index, NextIndex]:= -1;
  for i:= 0 to pSize - 1 do
      fQ[i, Index]:= -1;
end;

function THeuristic.Run: Boolean;
var
  i, k: Integer;
  Res: Boolean;
begin
  pP[1]:= 0;
  i:= 1;
  k:= 0;
  Result:= False;
  Res:= False;
  while (i < pSize) do
  begin
    if (pForceStop) then
      Exit;
    Res:= SmallerDistance(k, pP[i]);
    if (not Res) then
    begin
      pPath:= E_PATH;
      pCost:= 0;
      Result:= False;
      Break;
    end;
    Inc(pFlop);
    Inc(pCost, pGraph.GetValue(k, pP[i]));
    k:= pP[i];
    Inc(i);
  end;
  if (Res) then
    if (pGraph.GetValue(pP[i - 1], 0) <> -1) then
    begin
      Inc(pFlop);
      Inc(pCost, pGraph.GetValue(pP[i - 1], 0));
      pP[i]:= 0;
      SetPath;
      Result:= True;
    end
    else
    begin
      pPath:= E_PATH;
      pCost:= 0;
      Result:= False;
    end;
  Synchronize(FireFound);
end;

end.

