//  Unit of TSP Solver
//  Created by GrMikeD, 2000-2003
//  http://kickme.to/GrMikeD
//  GrMikeD@Freemail.gr
unit MainFrm;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls, ComCtrls, Buttons, Grids, TeeProcs,
  TeEngine, Chart, jpeg, Series, Spin, Menus, ImgList;

type
  TMainForm = class(TForm)
    AlgorithmGroup: TRadioGroup;
    StatusBar: TStatusBar;
    StartButton: TBitBtn;
    LoadRandomButton: TBitBtn;
    FileButton: TBitBtn;
    ResultBox: TGroupBox;
    AlgoLabel: TLabel;
    DescLabel: TLabel;
    Bevel: TBevel;
    FlopEdit: TLabeledEdit;
    TimeEdit: TLabeledEdit;
    AddResultsButton: TBitBtn;
    AddAutoBox: TCheckBox;
    Panel: TPanel;
    PageControl: TPageControl;
    GraphSheet: TTabSheet;
    ResultsSheet: TTabSheet;
    TimeSheet: TTabSheet;
    FlopSheet: TTabSheet;
    CostSheet: TTabSheet;
    GraphGrid: TStringGrid;
    OptimalPathEdit: TLabeledEdit;
    CostEdit: TLabeledEdit;
    SaveTimeButton: TBitBtn;
    CopyTimeButton: TBitBtn;
    LogTimeBox: TCheckBox;
    LogFlopBox: TCheckBox;
    CopyFlopButton: TBitBtn;
    SaveFlopButton: TBitBtn;
    ViewCostBox: TCheckBox;
    CopyCostButton: TBitBtn;
    SaveCostButton: TBitBtn;
    Label8: TLabel;
    Label9: TLabel;
    ExhaustiveList: TListView;
    HeuristicList: TListView;
    ClearExhaustiveButton: TBitBtn;
    ClearHeuristicButton: TBitBtn;
    TitleLabel: TLabel;
    Label4: TLabel;
    WebSiteLabel: TLabel;
    Label5: TLabel;
    EmailLabel: TLabel;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Image: TImage;
    FlopChart: TChart;
    ExhaustiveFlopPoint: TPointSeries;
    HeuristicFlopPoint: TPointSeries;
    ExhaustiveFlopLine: TLineSeries;
    HeuristicFlopLine: TLineSeries;
    CostChart: TChart;
    ExhaustiveCostBar: TBarSeries;
    HeuristicCostBar: TBarSeries;
    TimeChart: TChart;
    ExhaustiveTimePoint: TPointSeries;
    HeuristicTimePoint: TPointSeries;
    ExhaustiveTimeLine: TLineSeries;
    HeuristicTimeLine: TLineSeries;
    NumberEdit: TSpinEdit;
    VerticesLabel: TLabel;
    OpenDialog: TOpenDialog;
    SaveDialog: TSaveDialog;
    FileMenu: TPopupMenu;
    FileLoadMenuItem: TMenuItem;
    FileSaveMenuItem: TMenuItem;
    ImageList: TImageList;
    ConnectedBox: TCheckBox;
    SaveHeuristicButton: TBitBtn;
    SaveExhaustiveButton: TBitBtn;
    procedure FormCreate(Sender: TObject);
    procedure NumberEditChange(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure LoadRandomButtonClick(Sender: TObject);
    procedure CopyTimeButtonClick(Sender: TObject);
    procedure SaveTimeButtonClick(Sender: TObject);
    procedure ClearHeuristicButtonClick(Sender: TObject);
    procedure LogTimeBoxClick(Sender: TObject);
    procedure ViewCostBoxClick(Sender: TObject);
    procedure StartButtonClick(Sender: TObject);
    procedure AddAutoBoxClick(Sender: TObject);
    procedure WebSiteLabelClick(Sender: TObject);
    procedure EmailLabelClick(Sender: TObject);
    procedure AddResultsButtonClick(Sender: TObject);
    procedure FileLoadMenuItemClick(Sender: TObject);
    procedure FileSaveMenuItemClick(Sender: TObject);
    procedure FileButtonClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure GraphGridDrawCell(Sender: TObject; ACol, ARow: Integer;
      Rect: TRect; State: TGridDrawState);
    procedure SaveHeuristicButtonClick(Sender: TObject);
  private
    { Private declarations }
    procedure ResetGraphGrid(const n: Integer);
    procedure FillGraphGrid;
    procedure SetItemsEnabled(const b: Boolean);
    procedure OnMyAlgoFound(Path: string; Cost: Integer);
    procedure OnMyAlgoEnd(Success: Boolean; Time: string; Flop: Integer);
    procedure OnMyAlgoTerminate(Sender: TObject);
    function SaveList(var List: TListView; const FTitle, FName: string): Integer;
  public
    { Public declarations }
  end;

var
  MainForm: TMainForm;

implementation

uses
  ShellAPI, Graph, Error, Algorithm, Exhaustive, Heuristic;

const
  MIN_V = 4;
  MAX_V = 200;
  APPEND_WORKING = ' - Working...';
  APP_TITLE = 'TSP Solver';

var
  MyGraph: TGraph;
  MyAlgorithm: TAlgorithm;
  MyFlop: Integer;
  MyTime: string;
  MyPath: string;
  MyCost: Integer;
  PathFound: Boolean;
  Interrupted: Boolean;

{$R *.dfm}

function  GetAppVersion:string;
var Size, Size2: DWord;
    Pt, Pt2: Pointer;
begin
  Size := GetFileVersionInfoSize(PChar(ParamStr(0)), Size2);
  if Size > 0 then
  begin
    GetMem(Pt, Size);
    try
      GetFileVersionInfo(PChar(ParamStr(0)), 0, Size, Pt);
      VerQueryValue(Pt, '\', Pt2, Size2);
      with TVSFixedFileInfo (Pt2^) do
      begin
        Result:= IntToStr(HiWord(dwFileVersionMS)) + '.' +
                 IntToStr(LoWord(dwFileVersionMS));
      end;
      finally
        FreeMem (Pt);
    end;
  end;
end;

procedure TMainForm.FormCreate(Sender: TObject);
begin
  Application.Title:= APP_TITLE;
  MainForm.Caption:= Application.Title;
  TitleLabel.Caption:= TitleLabel.Caption + ' ' + GetAppVersion;
  PageControl.ActivePageIndex:= 0;
  VerticesLabel.Caption:= Format('Number of Vertices (%d-%d):', [MIN_V, MAX_V]);
  NumberEdit.MinValue:= MIN_V;
  NumberEdit.MaxValue:= MAX_V;
  NumberEdit.Value:= NumberEdit.MinValue;
  ResetGraphGrid(NumberEdit.Value);
  MyGraph:= TGraph.Create(NumberEdit.Value);
  PathFound:= False;
  Interrupted:= False;
end;

procedure TMainForm.FormDestroy(Sender: TObject);
begin
  MyGraph.Free;
end;

procedure TMainForm.ResetGraphGrid(const n: Integer);
var
  i: Integer;
begin
  GraphGrid.ColCount:= n + 1;
  GraphGrid.RowCount:= GraphGrid.ColCount;
  for i:= 1 to GraphGrid.RowCount-1 do
  begin
    GraphGrid.Rows[i].Text:= 'a' + IntToStr(i);
    GraphGrid.Cols[i].Text:= GraphGrid.Rows[i].Text;
  end;
end;

procedure TMainForm.NumberEditChange(Sender: TObject);
begin
  FileSaveMenuItem.Enabled:= False;
  StartButton.Enabled:= False;
  ResetGraphGrid(NumberEdit.Value);
end;

procedure TMainForm.LoadRandomButtonClick(Sender: TObject);
begin
  Screen.Cursor:= crHourGlass;
  LoadRandomButton.Enabled:= False;
  MyGraph.Size:= NumberEdit.Value;
  MyGraph.LoadRandomValues(ConnectedBox.Checked);
  FillGraphGrid;
  LoadRandomButton.Enabled:= True;
  LoadRandomButton.SetFocus;
  Screen.Cursor:= crDefault;
end;

procedure TMainForm.FillGraphGrid;
var
  i, j: Integer;
begin
  for i:= 1 to GraphGrid.RowCount - 1 do
    for j:= 1 to GraphGrid.ColCount - 1 do
      GraphGrid.Cells[j, i]:= IntToStr(MyGraph.GetValue(i - 1, j - 1));
  StartButton.Enabled:= True;
  FileSaveMenuItem.Enabled:= True;
end;

procedure TMainForm.CopyTimeButtonClick(Sender: TObject);
begin
  if (Sender = CopyTimeButton) then
    TimeChart.CopyToClipboardBitmap
  else if (Sender = CopyFlopButton) then
    FlopChart.CopyToClipboardBitmap
  else if (Sender = CopyCostButton) then
    CostChart.CopyToClipboardBitmap;
end;

procedure TMainForm.SaveTimeButtonClick(Sender: TObject);
begin
  if (Sender = SaveTimeButton) then
    SaveDialog.FileName:= 'timeN.bmp'
  else if (Sender = SaveFlopButton) then
    SaveDialog.FileName:= 'flopN.bmp'
  else if (Sender = SaveCostButton) then
    SaveDialog.FileName:= 'costN.bmp'
  else
    Exit;
  SaveDialog.DefaultExt:= 'bmp';
  SaveDialog.Filter:= 'Bitmap Files (*.bmp)|*.bmp|Meta Files (*.wmf)|*.wmf|All Files (*.*)|*.*';
  SaveDialog.FilterIndex:= 0;
  if SaveDialog.Execute then
  begin
    if ExtractFileExt(SaveDialog.FileName) = '.wmf' then
    begin
      if (Sender = SaveTimeButton) then
        TimeChart.SaveToMetafile(SaveDialog.FileName)
      else if (Sender = SaveFlopButton) then
        FlopChart.SaveToMetafile(SaveDialog.FileName)
      else if (Sender = SaveCostButton) then
        CostChart.SaveToMetafile(SaveDialog.FileName);
    end
    else
    begin
      if (Sender = SaveTimeButton) then
        TimeChart.SaveToBitmapFile(SaveDialog.FileName)
      else if (Sender = SaveFlopButton) then
        FlopChart.SaveToBitmapFile(SaveDialog.FileName)
      else if (Sender = SaveCostButton) then
        CostChart.SaveToBitmapFile(SaveDialog.FileName);
    end;
  end;
end;

procedure TMainForm.ClearHeuristicButtonClick(Sender: TObject);
begin
  if (Sender = ClearExhaustiveButton) then
  begin
    ExhaustiveList.Items.Clear;
    ClearExhaustiveButton.Enabled:= False;
    SaveExhaustiveButton.Enabled:= False;
    ExhaustiveFlopLine.Clear;
    ExhaustiveFlopPoint.Clear;
    ExhaustiveTimeLine.Clear;
    ExhaustiveTimePoint.Clear;
    ExhaustiveCostBar.Clear;
  end
  else
    if (Sender = ClearHeuristicButton) then
    begin
      HeuristicList.Items.Clear;
      ClearHeuristicButton.Enabled:= False;
      SaveHeuristicButton.Enabled:= False;
      HeuristicFlopLine.Clear;
      HeuristicFlopPoint.Clear;
      HeuristicTimeLine.Clear;
      HeuristicTimePoint.Clear;
      HeuristicCostBar.Clear;
    end;
end;

procedure TMainForm.LogTimeBoxClick(Sender: TObject);
begin
  if (Sender = LogTimeBox) then
    TimeChart.LeftAxis.Logarithmic:= LogTimeBox.Checked
  else if (Sender = LogFlopBox) then
    FlopChart.LeftAxis.Logarithmic:= LogFlopBox.Checked;
end;

procedure TMainForm.ViewCostBoxClick(Sender: TObject);
begin
  CostChart.View3D:= not(CostChart.View3D);
end;

procedure TMainForm.StartButtonClick(Sender: TObject);
begin
  if (StartButton.Caption = '&Cancel') then
  begin
    MyAlgorithm.ForceStop;
    Interrupted:= True;
    Exit;
  end;
  Screen.Cursor:= crHourGlass;
  Application.Title:= APP_TITLE + APPEND_WORKING;
  MainForm.Caption:= Application.Title;
  StartButton.Enabled:= False;
  AddResultsButton.Enabled:= False;
  PageControl.ActivePageIndex:= 0;
  case AlgorithmGroup.ItemIndex of
    0:
      begin
        MyAlgorithm:= THeuristic.Create(MyGraph, True);
        AlgoLabel.Caption:= 'Heuristic Algorithm';
      end;
    1:
      begin
        MyAlgorithm:= TExhaustive.Create(MyGraph, True);
        AlgoLabel.Caption:= 'Exhaustive Algorithm';
      end;
  end;
  DescLabel.Caption:= 'For ' + IntToStr(NumberEdit.Value) + ' Nodes';
  FlopEdit.Text:= '0';
  TimeEdit.Text:= '0 sec';
  MyAlgorithm.OnFound:= OnMyAlgoFound;
  MyAlgorithm.OnEnd:= OnMyAlgoEnd;
  MyAlgorithm.OnTerminate:= OnMyAlgoTerminate;
  MyAlgorithm.Resume;
  SetItemsEnabled(False);
  StartButton.Enabled:= True;
  Screen.Cursor:= crDefault;
end;

procedure TMainForm.OnMyAlgoTerminate(Sender: TObject);
begin
  Screen.Cursor:= crHourGlass;
  StartButton.Enabled:= False;
  if (not Interrupted) then
  begin
    TimeEdit.Text:= MyTime;
    FlopEdit.Text:= IntToStr(MyFlop);
  end
  else
  begin
    OptimalPathEdit.Text:= E_INTERRUPTED;
    CostEdit.Text:= '0';
  end;
  MyAlgorithm:= nil;
  SetItemsEnabled(True);
  if (AddAutoBox.Checked) then
  begin
    if (PathFound) then
      AddResultsButtonClick(AddResultsButton);
  end
  else
    AddResultsButton.Enabled:= PathFound;
  if Interrupted then
    AddResultsButton.Enabled:= False;
  StartButton.Enabled:= True;
  Interrupted:= False;
  Application.Title:= APP_TITLE;
  MainForm.Caption:= Application.Title;
  Screen.Cursor:= crDefault;
end;

procedure TMainForm.SetItemsEnabled(const b: Boolean);
begin
  AlgorithmGroup.Enabled:= b;
  NumberEdit.Enabled:= b;
  LoadRandomButton.Enabled:= b;
  ConnectedBox.Enabled:= b;
  FileButton.Enabled:= b;
  if (b) then
  begin
    StartButton.Kind:= bkOK;
    StartButton.Caption:= '&Start';
  end
  else
  begin
    StartButton.Kind:= bkCancel;
    StartButton.Caption:= '&Cancel';
  end;
  AddAutoBox.Enabled:= b;
  OptimalPathEdit.Enabled:= b;
  CostEdit.Enabled:= b;
end;

procedure TMainForm.AddAutoBoxClick(Sender: TObject);
begin
  if (PathFound) then
    AddResultsButton.Enabled:= not AddAutoBox.Checked;
end;

procedure TMainForm.OnMyAlgoFound(Path: string; Cost: Integer);
begin
  MyPath:= Path;
  MyCost:= Cost;
  OptimalPathEdit.Text:= MyPath;
  OptimalPathEdit.Update;
  CostEdit.Text:= IntToStr(MyCost);
  CostEdit.Update;
end;


procedure TMainForm.WebSiteLabelClick(Sender: TObject);
begin
  ShellExecute(Handle, 'open', PChar('http://kickme.to/GrMikeD'), nil, nil, SW_SHOW);
end;

procedure TMainForm.EmailLabelClick(Sender: TObject);
begin
  ShellExecute(Handle, 'open', PChar('mailto:GrMikeD@Freemail.gr?subject=For the TSP Solver'), nil, nil, SW_SHOW);
end;

procedure TMainForm.AddResultsButtonClick(Sender: TObject);
var
  i: integer;
  Item, FoundItem: TListItem;
  V: Integer;
begin
  V:= NumberEdit.Value;
  if (AlgorithmGroup.ItemIndex = 0) then
  begin
    for i:= 0 to HeuristicFlopPoint.XValues.Count-1 do
    begin
        if (HeuristicFlopPoint.XValues[i] = V) then
        begin
           HeuristicFlopPoint.Delete(i);
           HeuristicTimePoint.Delete(i);
           HeuristicCostBar.Delete(i);
           break;
        end;
    end;
    HeuristicFlopPoint.AddXY(V, MyFlop, '', clBlue);
    HeuristicTimePoint.AddXY(V, StrToFloat(MyTime), '', clBlue);
    HeuristicCostBar.AddXY(V, MyCost, '', clBlue);
    FoundItem:= HeuristicList.FindCaption(0, IntToStr(V), False, True, False);
    if (FoundItem <> nil) then
      FoundItem.Delete;
    Item:= HeuristicList.Items.Add;
    Item.Caption:= IntToStr(V);
    Item.SubItems.Add(IntToStr(MyFlop));
    Item.SubItems.Add(MyTime);
    Item.SubItems.Add(IntToStr(MyCost));
    ClearHeuristicButton.Enabled:= True;
    SaveHeuristicButton.Enabled:= True;
  end
  else
  begin
    for i:= 0 to ExhaustiveFlopPoint.XValues.Count - 1 do
    begin
        if (ExhaustiveFlopPoint.XValues[i] = V) then
        begin
           ExhaustiveFlopPoint.Delete(i);
           ExhaustiveTimePoint.Delete(i);
           ExhaustiveCostBar.Delete(i);
           break;
        end;
    end;
    ExhaustiveFlopPoint.AddXY(V, MyFlop, '', clRed);
    ExhaustiveTimePoint.AddXY(V, StrToFloat(MyTime), '', clRed);
    ExhaustiveCostBar.AddXY(V, MyCost, '', clRed);
    FoundItem:= ExhaustiveList.FindCaption(0, IntToStr(V), False, True, False);
    if (FoundItem <> nil) then
      FoundItem.Delete;
    Item:= ExhaustiveList.Items.Add;
    Item.Caption:= IntToStr(V);
    Item.SubItems.Add(IntToStr(MyFlop));
    Item.SubItems.Add(MyTime);
    Item.SubItems.Add(IntToStr(MyCost));
    ClearExhaustiveButton.Enabled:= True;
    SaveExhaustiveButton.Enabled:= True;
  end;
end;

procedure TMainForm.OnMyAlgoEnd(Success: Boolean; Time: string; Flop: Integer);
begin
  MyTime:= Time;
  MyFlop:= Flop;
  PathFound:= Success;
end;

procedure TMainForm.FileLoadMenuItemClick(Sender: TObject);
var
  Code: Integer;
begin
  Screen.Cursor:= crHourGlass;
  FileButton.Enabled:= False;
  if (OpenDialog.Execute) then
  begin
    MyGraph.Size:= NumberEdit.Value;
    Code:= MyGraph.LoadValuesFromFile(OpenDialog.FileName);
    if (Code = E_NOERROR) then
      FillGraphGrid
    else
    begin
      Screen.Cursor:= crDefault;
      Beep;
      MessageDlg(Format('Error: %s', [ErrorMessage(code)]), MtError, [MbOK], 0);
    end;
  end;
  FileButton.Enabled:= True;
  FileButton.SetFocus;
  Screen.Cursor:= crDefault;
end;

procedure TMainForm.FileSaveMenuItemClick(Sender: TObject);
var
  Code: Integer;
begin
  Screen.Cursor:= crHourGlass;
  FileButton.Enabled:= False;
  SaveDialog.DefaultExt:= 'txt';
  SaveDialog.Filter:= 'Text Files (*.txt)|*.txt|All Files (*.*)|*.*';
  SaveDialog.FilterIndex:= 0;
  SaveDialog.FileName:= 'TSP(' + IntTostr(MyGraph.Size) + ').txt';
  if (SaveDialog.Execute) then
  begin
    Code:= MyGraph.SaveValuesToFile(SaveDialog.FileName);
    if (Code <> E_NOERROR) then
    begin
      Screen.Cursor:= crDefault;
      Beep;
      MessageDlg(Format('Error: %s', [ErrorMessage(code)]), MtError, [MbOK], 0);
    end;
  end;
  FileButton.Enabled:= True;
  FileButton.SetFocus;
  Screen.Cursor:= crDefault;
end;

procedure TMainForm.FileButtonClick(Sender: TObject);
var
  P: TPoint;
  TheMsg: TMsg;
begin
  FileButton.Invalidate;
  P := FileButton.ClientToScreen(Point(0, FileButton.Height));
  FileMenu.Popup(P.X, P.Y);
  while PeekMessage(TheMsg, HWND(0), WM_MOUSEFIRST, WM_MOUSELAST, PM_REMOVE) do
    ;
  if GetCapture <> 0 then
    SendMessage(GetCapture, WM_CANCELMODE, 0, 0);
  FileButton.Invalidate;
end;

procedure TMainForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  if (MyAlgorithm <> nil) then
    MyAlgorithm.ForceStop;
end;

procedure TMainForm.GraphGridDrawCell(Sender: TObject; ACol, ARow: Integer;
  Rect: TRect; State: TGridDrawState);
begin
  if (ACol = ARow) and (ACol > 0) then
  begin
    GraphGrid.Canvas.Brush.Color:= clBtnFace;
    GraphGrid.Canvas.FillRect(Rect);
  end;
end;

function TMainForm.SaveList(var List: TListView; const FTitle, FName: string): Integer;
var
  F: TextFile;
  i, n: Integer;
  Item: TListitem;
begin
  Result:= E_NOERROR;
  AssignFile(F, FName);
  try
    FileMode:=  fmOpenWrite;
    Rewrite(F);
    n:= List.Items.Count;
    if (n > 0) then
    begin
      Write(F, FTitle + #13#10#13#10#13#10);
      Write(F, 'N'#9#9'Flop'#9#9'Time (sec)'#9#9'Cost'#13#10#13#10);
      for i:= 0 to n - 1 do
      begin
        Item:= List.Items[i];
        Write(F, Format('%s'#9#9'%s'#9#9'%s'#9#9#9'%s', [Item.Caption, Item.SubItems[0], Item.SubItems[1], Item.SubItems[2]]));
        if (i < n - 1) then
          Write(F, #13#10);
      end;
    end;
    CloseFile(F);
    except
      Result:= E_FILESAVE;
  end;
end;

procedure TMainForm.SaveHeuristicButtonClick(Sender: TObject);
var
  Code: Integer;
begin
  Screen.Cursor:= crHourGlass;
  (Sender as TBitBtn).Enabled:= False;
  if (Sender = SaveHeuristicButton) then
    SaveDialog.FileName:= 'Heuristic.txt'
  else
    SaveDialog.FileName:= 'Exhaustive.txt';
  SaveDialog.DefaultExt:= 'txt';
  SaveDialog.Filter:= 'Text Files (*.txt)|*.txt|All Files (*.*)|*.*';
  SaveDialog.FilterIndex:= 0;
  if (SaveDialog.Execute) then
  begin
    Code:= E_NOERROR;
    if (Sender = SaveHeuristicButton) then
      Code:= SaveList(HeuristicList, 'Heuristic Algorithm', SaveDialog.FileName)
    else
      if (Sender = SaveExhaustiveButton) then
        Code:= SaveList(ExhaustiveList, 'Exhaustive Algorithm', SaveDialog.FileName);
    if (Code <> E_NOERROR) then
    begin
      Screen.Cursor:= crDefault;
      Beep;
      MessageDlg(Format('Error: %s', [ErrorMessage(code)]), MtError, [MbOK], 0);
    end;
  end;
  (Sender as TBitBtn).Enabled:= True;
  Screen.Cursor:= crDefault;
end;

end.
